@@ -133,23 +133,30 @@ class CalibrationController(object):
         return
 
     def find_peaks(self, calib_key, min_prominence=15,
-                   num_brillouin_samples=2):
+                   num_brillouin_samples=2, min_height=15):
         spectra = self.extract_calibration_spectra(calib_key)
         if spectra is None:
             return
         spectrum = np.mean(spectra, axis=0)
+        # This is the background value
+        base = np.nanmedian(spectrum)
         peaks, properties = find_peaks(
-            spectrum, prominence=min_prominence, width=True)
+            spectrum, prominence=min_prominence, width=True,
+            height=min_height+base)
 
         # Number of peaks we are searching for
         # (2 Rayleigh + 2 times number calibration samples)
-        num_peaks_brillouin = 2 * num_brillouin_samples
-        num_peaks_rayleigh = 2
-        num_peaks = num_peaks_rayleigh + num_peaks_brillouin
+        num_peaks = 2 + 2 * num_brillouin_samples
 
         # Check if we found enough peak candidates
         if len(peaks) < num_peaks:
-            return
+            # If we didn't find enough peaks, we try again
+            # without a minimum height
+            peaks, properties = find_peaks(
+                spectrum, prominence=min_prominence, width=True)
+            # If there a still too few, we give up
+            if len(peaks) < num_peaks:
+                return
 
         # We need to identify the position between the
         # Stokes and Anti-Stokes Brillouin peaks
@@ -161,8 +168,9 @@ class CalibrationController(object):
             center = np.mean(peaks[idx - 1:idx + 1])
         # Otherwise we use the center of mass as the middle
         else:
-            # Subtract background value so it does not affect the center
-            spectrum = spectrum - np.nanmedian(spectrum)
+            # Set everything below the background value to zero,
+            # so it does not affect the center calculation
+            spectrum[spectrum < base] = 0
             # Calculate the center of mass
             center = np.nansum(spectrum * range(1, len(spectrum) + 1))\
                 / np.nansum(spectrum)
@@ -174,12 +182,12 @@ class CalibrationController(object):
             # If not enough peaks on the right, shift center to the left
             if num_peaks_right < (num_brillouin_samples + 1):
                 center = np.mean(
-                    peaks[num_brillouin_samples:num_brillouin_samples+2]
+                    peaks[-(num_brillouin_samples + 2):-num_brillouin_samples]
                 )
             # If not enough peaks on the left, shift center to the right
             elif num_peaks_left < (num_brillouin_samples + 1):
                 center = np.mean(
-                    peaks[-(num_brillouin_samples+2):-num_brillouin_samples]
+                    peaks[num_brillouin_samples:num_brillouin_samples + 2]
                 )
 
         num_peaks_left = len(peaks[peaks <= center])
@@ -477,6 +485,7 @@ class EvaluationController(object):
                     packed_data = zip(irepeat(spectra), regions)
                     # Process it
                     results = pool.starmap(self.fit_spectra, packed_data)
+
                     # Unpack the results
                     for frame_num, spectrum in enumerate(spectra):
                         for region_key, region in enumerate(brillouin_regions):
@@ -501,6 +510,55 @@ class EvaluationController(object):
                             evm.results['rayleigh_peak_intensity'][ind] =\
                                 results[region_key][frame_num][2]
 
+                    # We can only do a multi-peak fit after the single-peak
+                    # Rayleigh fit is done, because we have to know the
+                    # Rayleigh peak positions in GHz in order to convert
+                    # the multi-peak fit bounds given in GHz into the position
+                    # in pixels.
+                    if evm.nr_brillouin_peaks > 1:
+                        ind =\
+                            (ind_x, ind_y, ind_z, slice(None), slice(None), 0)
+                        rayleigh_peaks = np.transpose(
+                            evm.results['rayleigh_peak_position'][ind]
+                        )
+                        bounds = self.create_bounds(
+                            brillouin_regions,
+                            times,
+                            rayleigh_peaks
+                        )
+                        if bounds is not None:
+                            packed_data_multi_peak =\
+                                zip(irepeat(spectra), brillouin_regions,
+                                    irepeat(evm.nr_brillouin_peaks),
+                                    bounds)
+                        else:
+                            packed_data_multi_peak =\
+                                zip(irepeat(spectra), brillouin_regions,
+                                    irepeat(evm.nr_brillouin_peaks),
+                                    irepeat(bounds))
+                        # Process it
+                        results_multi_peak = pool.starmap(
+                            self.fit_spectra, packed_data_multi_peak)
+
+                        for frame_num, spectrum in enumerate(spectra):
+                            for region_key, region in enumerate(
+                                    brillouin_regions):
+                                ind = (ind_x, ind_y, ind_z,
+                                       frame_num, region_key,
+                                       slice(1, evm.nr_brillouin_peaks+1))
+                                evm.results[
+                                    'brillouin_peak_position'][ind] = \
+                                    results_multi_peak[
+                                        region_key][frame_num][0]
+                                evm.results[
+                                    'brillouin_peak_fwhm'][ind] = \
+                                    results_multi_peak[
+                                        region_key][frame_num][1]
+                                evm.results[
+                                    'brillouin_peak_intensity'][ind] = \
+                                    results_multi_peak[
+                                        region_key][frame_num][2]
+
                     if count is not None:
                         count.value += 1
 
@@ -516,14 +574,99 @@ class EvaluationController(object):
         return
 
     @staticmethod
-    def fit_spectra(spectra, region):
+    def fit_spectra(spectra, region, nr_peaks=1, bounds_w0=None):
         fits = []
         for frame_num, spectrum in enumerate(spectra):
             xdata = np.arange(len(spectrum))
-            fit = fit_lorentz_region(region, xdata, spectrum)
+            if bounds_w0 is None:
+                fit = fit_lorentz_region(region, xdata, spectrum,
+                                         nr_peaks)
+            else:
+                fit = fit_lorentz_region(region, xdata, spectrum,
+                                         nr_peaks, bounds_w0[frame_num])
             fits.append(fit)
         return fits
 
+    def create_bounds(self, brillouin_regions, times, rayleigh_peaks):
+        """
+        This function converts the bounds settings into
+        a bounds object for the fitting function
+        Allowed parameters for the bounds settings are
+        - 'min'/'max'   -> Will be converted to the respective
+            lower or upper limit of the given region
+        - '-Inf', 'Inf' -> Will be converted to -np.Inf or np.Inf
+        - number [GHz]  -> Will be converted into the pixel
+            position of the given frequency
+        Parameters
+        ----------
+        brillouin_regions
+        times
+        rayleigh_peaks
+
+        Returns
+        -------
+
+        """
+        cm = self.session.calibration_model()
+        evm = self.session.evaluation_model()
+        bounds = evm.bounds
+        if bounds is None:
+            return None
+
+        # We need a Rayleigh peak position for
+        # every combination of brillouin_region and time
+        if rayleigh_peaks.shape != (len(brillouin_regions), len(times)):
+            return None
+
+        w0_bounds = []
+        # We have to create a separate bound for every region
+        for region_idx, region in enumerate(brillouin_regions):
+            local_time = []
+            for time_idx, time in enumerate(times):
+                # In case this is an Anti-Stokes peak, we find the peak
+                # with the higher frequency on the left hand side and
+                # have to flip the bounds
+                is_anti_stokes = np.mean(region) <\
+                                 rayleigh_peaks[region_idx][time_idx]
+                f_rayleigh = cm.get_frequency_by_time(
+                    time,
+                    rayleigh_peaks[region_idx][time_idx]
+                )[()]
+
+                local_bound = []
+                for bound in bounds:
+                    local_limit = []
+                    for limit in bound:
+                        if limit.lower() == 'min':
+                            val = region[is_anti_stokes]
+                        elif limit.lower() == 'max':
+                            val = region[not is_anti_stokes]
+                        elif limit.lower() == '-inf':
+                            val = -((-1) ** is_anti_stokes)\
+                                          * np.Inf
+                        elif limit.lower() == 'inf':
+                            val = ((-1) ** is_anti_stokes)\
+                                          * np.Inf
+                        else:
+                            # Try to convert the value in GHz into
+                            # a value in pixel depending on the time
+                            try:
+                                f = ((-1) ** is_anti_stokes)\
+                                          * 1e9 * float(limit) + f_rayleigh
+                                val = cm.get_position_by_time(time, f)[()]
+                            except BaseException:
+                                val = np.Inf
+
+                        local_limit.append(val)
+                    # Check that the bounds are sorted ascendingly
+                    # (for anti-stokes, they might not).
+                    local_limit.sort()
+                    local_bound.append(local_limit)
+                local_time.append(local_bound)
+            w0_bounds.append(local_time)
+
+        return w0_bounds
+
     def extract_payload_spectra(self, image_key):
         em = self.session.extraction_model()
         evm = self.session.evaluation_model()
@@ -554,7 +697,7 @@ class EvaluationController(object):
         evm.set_spectra(image_key, spectra)
         return spectra, times, intensities
 
-    def get_data(self, parameter_key):
+    def get_data(self, parameter_key, brillouin_peak_index=0):
         """
         This function returns the evaluated data,
         its positions, dimensionality and labels
@@ -563,6 +706,13 @@ class EvaluationController(object):
         parameter_key: str
             The key of the parameter requested.
             See bmlab.model.evaluation_model.get_parameter_keys()
+        brillouin_peak_index: int
+            The index of the Brillouin peak to show in case
+            we did a multi-peak fit
+            0: the single-peak fit
+            1:nr_brillouin_peaks: the multi-peak fits
+            nr_brillouin_peaks+1: all multi-peak fits average
+            nr_brillouin_peaks+1: all multi-peak fits weighted average
 
         Returns
         -------
@@ -595,6 +745,30 @@ class EvaluationController(object):
             data = np.empty(resolution)
             data[:] = np.nan
 
+        # Slice the appropriate Brillouin peak if necessary and possible
+        if data.ndim >= 6:
+            nr_peaks_stored = data.shape[5]
+            if nr_peaks_stored > 1\
+                    and brillouin_peak_index < nr_peaks_stored + 2:
+                if brillouin_peak_index < nr_peaks_stored:
+                    sliced = data[:, :, :, :, :, brillouin_peak_index]
+                # Average all multi-peak fits
+                if brillouin_peak_index == nr_peaks_stored:
+                    sliced = data[:, :, :, :, :, 1:]
+                # Weighted average of all multi-peak fits
+                if brillouin_peak_index == nr_peaks_stored + 1:
+                    weight =\
+                        evm.results[
+                            'brillouin_peak_intensity'][:, :, :, :, :, 1:]\
+                        * evm.results['brillouin_peak_fwhm'][:, :, :, :, :, 1:]
+                    sliced = \
+                        np.nansum(data[:, :, :, :, :, 1:] * weight, axis=5)\
+                        / np.nansum(weight, axis=5)
+            else:
+                sliced = data[:, :, :, :, :, 0]
+        else:
+            sliced = data
+
         # Average all non-spatial dimensions.
         # Do not show warning which occurs when a slice contains only NaNs.
         with warnings.catch_warnings():
@@ -603,8 +777,8 @@ class EvaluationController(object):
                 message='Mean of empty slice'
             )
             data = np.nanmean(
-                data,
-                axis=tuple(range(3, data.ndim))
+                sliced,
+                axis=tuple(range(3, sliced.ndim))
             )
 
         # Scale the date in case of GHz
