@@ -49,7 +49,7 @@ def fit_lorentz(x, y):
     return w0, fwhm, intensity, offset
 
 
-def fit_double_lorentz(x, y):
+def fit_double_lorentz(x, y, bounds_w0=None):
     offset_guess = (y[0] + y[-1]) / 2.
     # gam_guess = (w0_guess ** 2 * (np.max(y) - offset_guess)) ** -0.5
     fwhm_guess = 4
@@ -65,8 +65,7 @@ def fit_double_lorentz(x, y):
         return
 
     idx_sort = np.sort(peaks[idx[0:2]])
-    w0_guess_left = x[idx_sort[0]]
-    w0_guess_right = x[idx_sort[1]]
+    w0_guess = list(x[idx_sort])
 
     def error(params, xdata, ydata):
         return (ydata
@@ -74,13 +73,61 @@ def fit_double_lorentz(x, y):
                 - lorentz(xdata, *params[3:6])
                 - params[6]) ** 2
 
+    # Create the bounds array
+    if bounds_w0 is not None:
+        # Lower limits
+        bounds_lower = -np.Inf * np.ones(7)
+
+        # 1st peak:
+        # central position
+        bounds_lower[0] = bounds_w0[0][0]
+        # full-width-half-maximum
+        # The VIPA spectrometer has an instrument width of
+        # approx. 750 MHz, which translates to around
+        # 7 px minimum peak width (FOB setup)
+        # and 180 MHz and 4 px (780 nm setup).
+        # We limit the minimal fit width to 3 px here.
+        bounds_lower[1] = 3
+        # intensity
+        bounds_lower[2] = 0
+
+        # 2nd peak:
+        # central position
+        bounds_lower[3] = bounds_w0[1][0]
+        # full-width-half-maximum
+        bounds_lower[4] = 3
+        # intensity
+        bounds_lower[5] = 0
+
+        # offset
+        bounds_lower[6] = 0
+
+        # Upper limits
+        bounds_upper = np.Inf * np.ones(7)
+
+        bounds_upper[0] = bounds_w0[0][1]
+        bounds_upper[3] = bounds_w0[1][1]
+        bounds = (bounds_lower, bounds_upper)
+
+        # Sort the guesses to the bounds
+        bounds_w0_center = [np.mean(
+            np.clip(bound, *x[::len(x) - 1])) for bound in bounds_w0]
+        w0_guess.sort(reverse=(bounds_w0_center[0] > bounds_w0_center[1]))
+
+        # Check that the initial guesses are within the bounds
+        w0_guess = [np.clip(
+            guess, *bounds_w0[idx]) for idx, guess in enumerate(w0_guess)]
+    else:
+        bounds = (-np.Inf, np.Inf)
+
     opt_result = least_squares(
         error,
-        x0=(w0_guess_left, fwhm_guess, intensity_guess,
-            w0_guess_right, fwhm_guess, intensity_guess,
+        x0=(w0_guess[0], fwhm_guess, intensity_guess,
+            w0_guess[1], fwhm_guess, intensity_guess,
             offset_guess
             ),
-        args=(x, y)
+        args=(x, y),
+        bounds=bounds
     )
 
     if not opt_result.success:
@@ -154,7 +201,7 @@ def _circle_opt(c, x_coord, y_coord):
          - c[2] ** 2) ** 2)
 
 
-def fit_lorentz_region(region, xdata, ydata, nr_peaks=1):
+def fit_lorentz_region(region, xdata, ydata, nr_peaks=1, bounds_w0=None):
     """
     Fits a lorentz or double lorentz fit to the given region
 
@@ -164,6 +211,7 @@ def fit_lorentz_region(region, xdata, ydata, nr_peaks=1):
     xdata: The x-data
     ydata: The y-data to fit
     nr_peaks: The number of peaks to fit
+    bounds_w0: The bounds for the lorentz fit value of the maximum position
 
     Returns
     -------
@@ -172,7 +220,8 @@ def fit_lorentz_region(region, xdata, ydata, nr_peaks=1):
     try:
         if nr_peaks == 2:
             w0s, fwhms, intensities, offset = fit_double_lorentz(
-                xdata[range(*region)], ydata[range(*region)])
+                xdata[range(*region)], ydata[range(*region)],
+                bounds_w0=bounds_w0)
         elif nr_peaks == 1:
             w0s, fwhms, intensities, offset = fit_lorentz(
                 xdata[range(*region)], ydata[range(*region)])
